"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
This Source Code Form is subject to the terms of the Mozilla Public
License, v. 2.0. If a copy of the MPL was not distributed with this
file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""

import builtins
import collections.abc
import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.duration_pb2
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.timestamp_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _ResourceType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _ResourceTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_ResourceType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    FEATURE: _ResourceType.ValueType  # 0
    LABEL: _ResourceType.ValueType  # 1
    TRAINING_SET: _ResourceType.ValueType  # 2
    SOURCE: _ResourceType.ValueType  # 3
    FEATURE_VARIANT: _ResourceType.ValueType  # 4
    LABEL_VARIANT: _ResourceType.ValueType  # 5
    TRAINING_SET_VARIANT: _ResourceType.ValueType  # 6
    SOURCE_VARIANT: _ResourceType.ValueType  # 7
    PROVIDER: _ResourceType.ValueType  # 8
    ENTITY: _ResourceType.ValueType  # 9
    MODEL: _ResourceType.ValueType  # 10
    USER: _ResourceType.ValueType  # 11

class ResourceType(_ResourceType, metaclass=_ResourceTypeEnumTypeWrapper): ...

FEATURE: ResourceType.ValueType  # 0
LABEL: ResourceType.ValueType  # 1
TRAINING_SET: ResourceType.ValueType  # 2
SOURCE: ResourceType.ValueType  # 3
FEATURE_VARIANT: ResourceType.ValueType  # 4
LABEL_VARIANT: ResourceType.ValueType  # 5
TRAINING_SET_VARIANT: ResourceType.ValueType  # 6
SOURCE_VARIANT: ResourceType.ValueType  # 7
PROVIDER: ResourceType.ValueType  # 8
ENTITY: ResourceType.ValueType  # 9
MODEL: ResourceType.ValueType  # 10
USER: ResourceType.ValueType  # 11
global___ResourceType = ResourceType

class _ComputationMode:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _ComputationModeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_ComputationMode.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    PRECOMPUTED: _ComputationMode.ValueType  # 0
    CLIENT_COMPUTED: _ComputationMode.ValueType  # 1
    STREAMING: _ComputationMode.ValueType  # 2

class ComputationMode(_ComputationMode, metaclass=_ComputationModeEnumTypeWrapper): ...

PRECOMPUTED: ComputationMode.ValueType  # 0
CLIENT_COMPUTED: ComputationMode.ValueType  # 1
STREAMING: ComputationMode.ValueType  # 2
global___ComputationMode = ComputationMode

class _ScalarType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _ScalarTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_ScalarType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    NULL: _ScalarType.ValueType  # 0
    INT: _ScalarType.ValueType  # 1
    FLOAT32: _ScalarType.ValueType  # 2
    FLOAT64: _ScalarType.ValueType  # 3
    STRING: _ScalarType.ValueType  # 4
    BOOL: _ScalarType.ValueType  # 5
    INT32: _ScalarType.ValueType  # 6
    INT64: _ScalarType.ValueType  # 7
    DATETIME: _ScalarType.ValueType  # 8

class ScalarType(_ScalarType, metaclass=_ScalarTypeEnumTypeWrapper): ...

NULL: ScalarType.ValueType  # 0
INT: ScalarType.ValueType  # 1
FLOAT32: ScalarType.ValueType  # 2
FLOAT64: ScalarType.ValueType  # 3
STRING: ScalarType.ValueType  # 4
BOOL: ScalarType.ValueType  # 5
INT32: ScalarType.ValueType  # 6
INT64: ScalarType.ValueType  # 7
DATETIME: ScalarType.ValueType  # 8
global___ScalarType = ScalarType

@typing.final
class OktaAuthConfig(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DOMAIN_FIELD_NUMBER: builtins.int
    AUTHORIZATION_SERVER_ID_FIELD_NUMBER: builtins.int
    CLIENT_ID_FIELD_NUMBER: builtins.int
    domain: builtins.str
    authorization_server_id: builtins.str
    client_id: builtins.str
    def __init__(
        self,
        *,
        domain: builtins.str = ...,
        authorization_server_id: builtins.str = ...,
        client_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["authorization_server_id", b"authorization_server_id", "client_id", b"client_id", "domain", b"domain"]) -> None: ...

global___OktaAuthConfig = OktaAuthConfig

@typing.final
class PassThroughAuthConfig(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___PassThroughAuthConfig = PassThroughAuthConfig

@typing.final
class AuthConfig(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OKTA_FIELD_NUMBER: builtins.int
    PASS_THROUGH_FIELD_NUMBER: builtins.int
    @property
    def okta(self) -> global___OktaAuthConfig: ...
    @property
    def pass_through(self) -> global___PassThroughAuthConfig: ...
    def __init__(
        self,
        *,
        okta: global___OktaAuthConfig | None = ...,
        pass_through: global___PassThroughAuthConfig | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["config", b"config", "okta", b"okta", "pass_through", b"pass_through"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["config", b"config", "okta", b"okta", "pass_through", b"pass_through"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["config", b"config"]) -> typing.Literal["okta", "pass_through"] | None: ...

global___AuthConfig = AuthConfig

@typing.final
class Name(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    name: builtins.str
    def __init__(
        self,
        *,
        name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["name", b"name"]) -> None: ...

global___Name = Name

@typing.final
class ErrorStatus(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CODE_FIELD_NUMBER: builtins.int
    MESSAGE_FIELD_NUMBER: builtins.int
    DETAILS_FIELD_NUMBER: builtins.int
    code: builtins.int
    """The status code, which should be an enum value of [google.rpc.Code][google.rpc.Code]."""
    message: builtins.str
    """A developer-facing error message, which should be in English. Any
    user-facing error message should be localized and sent in the
    [google.rpc.Status.details][google.rpc.Status.details] field, or localized by the client.
    """
    @property
    def details(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[google.protobuf.any_pb2.Any]:
        """A list of messages that carry the error details.  There will be a
        common set of message types for APIs to use.
        """

    def __init__(
        self,
        *,
        code: builtins.int = ...,
        message: builtins.str = ...,
        details: collections.abc.Iterable[google.protobuf.any_pb2.Any] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["code", b"code", "details", b"details", "message", b"message"]) -> None: ...

global___ErrorStatus = ErrorStatus

@typing.final
class ResourceStatus(google.protobuf.message.Message):
    """TODO: Deprecate this field in buildable resources when scheduling is fully implemented"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Status:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _StatusEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ResourceStatus._Status.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        NO_STATUS: ResourceStatus._Status.ValueType  # 0
        CREATED: ResourceStatus._Status.ValueType  # 1
        PENDING: ResourceStatus._Status.ValueType  # 2
        READY: ResourceStatus._Status.ValueType  # 3
        FAILED: ResourceStatus._Status.ValueType  # 4
        RUNNING: ResourceStatus._Status.ValueType  # 5
        CANCELLED: ResourceStatus._Status.ValueType  # 6

    class Status(_Status, metaclass=_StatusEnumTypeWrapper): ...
    NO_STATUS: ResourceStatus.Status.ValueType  # 0
    CREATED: ResourceStatus.Status.ValueType  # 1
    PENDING: ResourceStatus.Status.ValueType  # 2
    READY: ResourceStatus.Status.ValueType  # 3
    FAILED: ResourceStatus.Status.ValueType  # 4
    RUNNING: ResourceStatus.Status.ValueType  # 5
    CANCELLED: ResourceStatus.Status.ValueType  # 6

    STATUS_FIELD_NUMBER: builtins.int
    ERROR_MESSAGE_FIELD_NUMBER: builtins.int
    ERROR_STATUS_FIELD_NUMBER: builtins.int
    status: global___ResourceStatus.Status.ValueType
    error_message: builtins.str
    @property
    def error_status(self) -> global___ErrorStatus: ...
    def __init__(
        self,
        *,
        status: global___ResourceStatus.Status.ValueType = ...,
        error_message: builtins.str = ...,
        error_status: global___ErrorStatus | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["error_status", b"error_status"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["error_message", b"error_message", "error_status", b"error_status", "status", b"status"]) -> None: ...

global___ResourceStatus = ResourceStatus

@typing.final
class ResourceID(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESOURCE_FIELD_NUMBER: builtins.int
    RESOURCE_TYPE_FIELD_NUMBER: builtins.int
    resource_type: global___ResourceType.ValueType
    @property
    def resource(self) -> global___NameVariant: ...
    def __init__(
        self,
        *,
        resource: global___NameVariant | None = ...,
        resource_type: global___ResourceType.ValueType = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["resource", b"resource"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["resource", b"resource", "resource_type", b"resource_type"]) -> None: ...

global___ResourceID = ResourceID

@typing.final
class SetStatusRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESOURCE_ID_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    @property
    def resource_id(self) -> global___ResourceID: ...
    @property
    def status(self) -> global___ResourceStatus: ...
    def __init__(
        self,
        *,
        resource_id: global___ResourceID | None = ...,
        status: global___ResourceStatus | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["resource_id", b"resource_id", "status", b"status"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["resource_id", b"resource_id", "status", b"status"]) -> None: ...

global___SetStatusRequest = SetStatusRequest

@typing.final
class ScheduleChangeRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESOURCE_ID_FIELD_NUMBER: builtins.int
    SCHEDULE_FIELD_NUMBER: builtins.int
    schedule: builtins.str
    @property
    def resource_id(self) -> global___ResourceID: ...
    def __init__(
        self,
        *,
        resource_id: global___ResourceID | None = ...,
        schedule: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["resource_id", b"resource_id"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["resource_id", b"resource_id", "schedule", b"schedule"]) -> None: ...

global___ScheduleChangeRequest = ScheduleChangeRequest

@typing.final
class NameVariant(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    VARIANT_FIELD_NUMBER: builtins.int
    name: builtins.str
    variant: builtins.str
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        variant: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["name", b"name", "variant", b"variant"]) -> None: ...

global___NameVariant = NameVariant

@typing.final
class Empty(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___Empty = Empty

@typing.final
class Feature(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    DEFAULT_VARIANT_FIELD_NUMBER: builtins.int
    VARIANTS_FIELD_NUMBER: builtins.int
    name: builtins.str
    default_variant: builtins.str
    @property
    def status(self) -> global___ResourceStatus: ...
    @property
    def variants(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        status: global___ResourceStatus | None = ...,
        default_variant: builtins.str = ...,
        variants: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["status", b"status"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["default_variant", b"default_variant", "name", b"name", "status", b"status", "variants", b"variants"]) -> None: ...

global___Feature = Feature

@typing.final
class Columns(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ENTITY_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    TS_FIELD_NUMBER: builtins.int
    entity: builtins.str
    value: builtins.str
    ts: builtins.str
    def __init__(
        self,
        *,
        entity: builtins.str = ...,
        value: builtins.str = ...,
        ts: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["entity", b"entity", "ts", b"ts", "value", b"value"]) -> None: ...

global___Columns = Columns

@typing.final
class PythonFunction(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    QUERY_FIELD_NUMBER: builtins.int
    query: builtins.bytes
    def __init__(
        self,
        *,
        query: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["query", b"query"]) -> None: ...

global___PythonFunction = PythonFunction

@typing.final
class Stream(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OFFLINE_PROVIDER_FIELD_NUMBER: builtins.int
    offline_provider: builtins.str
    def __init__(
        self,
        *,
        offline_provider: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["offline_provider", b"offline_provider"]) -> None: ...

global___Stream = Stream

@typing.final
class ResourceVariant(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURE_VARIANT_FIELD_NUMBER: builtins.int
    LABEL_VARIANT_FIELD_NUMBER: builtins.int
    TRAINING_SET_VARIANT_FIELD_NUMBER: builtins.int
    SOURCE_VARIANT_FIELD_NUMBER: builtins.int
    @property
    def feature_variant(self) -> global___FeatureVariant: ...
    @property
    def label_variant(self) -> global___LabelVariant: ...
    @property
    def training_set_variant(self) -> global___TrainingSetVariant: ...
    @property
    def source_variant(self) -> global___SourceVariant: ...
    def __init__(
        self,
        *,
        feature_variant: global___FeatureVariant | None = ...,
        label_variant: global___LabelVariant | None = ...,
        training_set_variant: global___TrainingSetVariant | None = ...,
        source_variant: global___SourceVariant | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["feature_variant", b"feature_variant", "label_variant", b"label_variant", "resource", b"resource", "source_variant", b"source_variant", "training_set_variant", b"training_set_variant"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["feature_variant", b"feature_variant", "label_variant", b"label_variant", "resource", b"resource", "source_variant", b"source_variant", "training_set_variant", b"training_set_variant"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["resource", b"resource"]) -> typing.Literal["feature_variant", "label_variant", "training_set_variant", "source_variant"] | None: ...

global___ResourceVariant = ResourceVariant

@typing.final
class FeatureVariant(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    VARIANT_FIELD_NUMBER: builtins.int
    SOURCE_FIELD_NUMBER: builtins.int
    ENTITY_FIELD_NUMBER: builtins.int
    CREATED_FIELD_NUMBER: builtins.int
    OWNER_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    PROVIDER_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    TRAININGSETS_FIELD_NUMBER: builtins.int
    COLUMNS_FIELD_NUMBER: builtins.int
    FUNCTION_FIELD_NUMBER: builtins.int
    STREAM_FIELD_NUMBER: builtins.int
    LAST_UPDATED_FIELD_NUMBER: builtins.int
    SCHEDULE_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    PROPERTIES_FIELD_NUMBER: builtins.int
    MODE_FIELD_NUMBER: builtins.int
    TASK_ID_FIELD_NUMBER: builtins.int
    TASK_ID_LIST_FIELD_NUMBER: builtins.int
    ADDITIONAL_PARAMETERS_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    name: builtins.str
    variant: builtins.str
    entity: builtins.str
    owner: builtins.str
    description: builtins.str
    provider: builtins.str
    schedule: builtins.str
    mode: global___ComputationMode.ValueType
    task_id: builtins.str
    @property
    def source(self) -> global___NameVariant: ...
    @property
    def created(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def status(self) -> global___ResourceStatus: ...
    @property
    def trainingsets(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NameVariant]: ...
    @property
    def columns(self) -> global___Columns: ...
    @property
    def function(self) -> global___PythonFunction: ...
    @property
    def stream(self) -> global___Stream: ...
    @property
    def last_updated(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def tags(self) -> global___Tags: ...
    @property
    def properties(self) -> global___Properties: ...
    @property
    def task_id_list(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    @property
    def additional_parameters(self) -> global___FeatureParameters: ...
    @property
    def type(self) -> global___ValueType: ...
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        variant: builtins.str = ...,
        source: global___NameVariant | None = ...,
        entity: builtins.str = ...,
        created: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        owner: builtins.str = ...,
        description: builtins.str = ...,
        provider: builtins.str = ...,
        status: global___ResourceStatus | None = ...,
        trainingsets: collections.abc.Iterable[global___NameVariant] | None = ...,
        columns: global___Columns | None = ...,
        function: global___PythonFunction | None = ...,
        stream: global___Stream | None = ...,
        last_updated: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        schedule: builtins.str = ...,
        tags: global___Tags | None = ...,
        properties: global___Properties | None = ...,
        mode: global___ComputationMode.ValueType = ...,
        task_id: builtins.str = ...,
        task_id_list: collections.abc.Iterable[builtins.str] | None = ...,
        additional_parameters: global___FeatureParameters | None = ...,
        type: global___ValueType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["additional_parameters", b"additional_parameters", "columns", b"columns", "created", b"created", "function", b"function", "last_updated", b"last_updated", "location", b"location", "properties", b"properties", "source", b"source", "status", b"status", "stream", b"stream", "tags", b"tags", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["additional_parameters", b"additional_parameters", "columns", b"columns", "created", b"created", "description", b"description", "entity", b"entity", "function", b"function", "last_updated", b"last_updated", "location", b"location", "mode", b"mode", "name", b"name", "owner", b"owner", "properties", b"properties", "provider", b"provider", "schedule", b"schedule", "source", b"source", "status", b"status", "stream", b"stream", "tags", b"tags", "task_id", b"task_id", "task_id_list", b"task_id_list", "trainingsets", b"trainingsets", "type", b"type", "variant", b"variant"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["location", b"location"]) -> typing.Literal["columns", "function", "stream"] | None: ...

global___FeatureVariant = FeatureVariant

@typing.final
class ValueType(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SCALAR_FIELD_NUMBER: builtins.int
    VECTOR_FIELD_NUMBER: builtins.int
    scalar: global___ScalarType.ValueType
    @property
    def vector(self) -> global___VectorType: ...
    def __init__(
        self,
        *,
        scalar: global___ScalarType.ValueType = ...,
        vector: global___VectorType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["Type", b"Type", "scalar", b"scalar", "vector", b"vector"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["Type", b"Type", "scalar", b"scalar", "vector", b"vector"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["Type", b"Type"]) -> typing.Literal["scalar", "vector"] | None: ...

global___ValueType = ValueType

@typing.final
class VectorType(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SCALAR_FIELD_NUMBER: builtins.int
    DIMENSION_FIELD_NUMBER: builtins.int
    IS_EMBEDDING_FIELD_NUMBER: builtins.int
    scalar: global___ScalarType.ValueType
    dimension: builtins.int
    is_embedding: builtins.bool
    def __init__(
        self,
        *,
        scalar: global___ScalarType.ValueType = ...,
        dimension: builtins.int = ...,
        is_embedding: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["dimension", b"dimension", "is_embedding", b"is_embedding", "scalar", b"scalar"]) -> None: ...

global___VectorType = VectorType

@typing.final
class FeatureParameters(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PRECOMPUTED_FIELD_NUMBER: builtins.int
    ONDEMAND_FIELD_NUMBER: builtins.int
    @property
    def precomputed(self) -> global___PrecomputedFeatureParameters: ...
    @property
    def ondemand(self) -> global___OndemandFeatureParameters: ...
    def __init__(
        self,
        *,
        precomputed: global___PrecomputedFeatureParameters | None = ...,
        ondemand: global___OndemandFeatureParameters | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["feature_type", b"feature_type", "ondemand", b"ondemand", "precomputed", b"precomputed"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["feature_type", b"feature_type", "ondemand", b"ondemand", "precomputed", b"precomputed"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["feature_type", b"feature_type"]) -> typing.Literal["precomputed", "ondemand"] | None: ...

global___FeatureParameters = FeatureParameters

@typing.final
class PrecomputedFeatureParameters(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___PrecomputedFeatureParameters = PrecomputedFeatureParameters

@typing.final
class OndemandFeatureParameters(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DEFINITION_FIELD_NUMBER: builtins.int
    definition: builtins.str
    def __init__(
        self,
        *,
        definition: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["definition", b"definition"]) -> None: ...

global___OndemandFeatureParameters = OndemandFeatureParameters

@typing.final
class FeatureLag(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURE_FIELD_NUMBER: builtins.int
    VARIANT_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    LAG_FIELD_NUMBER: builtins.int
    feature: builtins.str
    variant: builtins.str
    name: builtins.str
    @property
    def lag(self) -> google.protobuf.duration_pb2.Duration: ...
    def __init__(
        self,
        *,
        feature: builtins.str = ...,
        variant: builtins.str = ...,
        name: builtins.str = ...,
        lag: google.protobuf.duration_pb2.Duration | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["lag", b"lag"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["feature", b"feature", "lag", b"lag", "name", b"name", "variant", b"variant"]) -> None: ...

global___FeatureLag = FeatureLag

@typing.final
class Label(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    DEFAULT_VARIANT_FIELD_NUMBER: builtins.int
    VARIANTS_FIELD_NUMBER: builtins.int
    name: builtins.str
    default_variant: builtins.str
    @property
    def status(self) -> global___ResourceStatus: ...
    @property
    def variants(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        status: global___ResourceStatus | None = ...,
        default_variant: builtins.str = ...,
        variants: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["status", b"status"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["default_variant", b"default_variant", "name", b"name", "status", b"status", "variants", b"variants"]) -> None: ...

global___Label = Label

@typing.final
class LabelVariant(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    VARIANT_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    SOURCE_FIELD_NUMBER: builtins.int
    ENTITY_FIELD_NUMBER: builtins.int
    CREATED_FIELD_NUMBER: builtins.int
    OWNER_FIELD_NUMBER: builtins.int
    PROVIDER_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    TRAININGSETS_FIELD_NUMBER: builtins.int
    COLUMNS_FIELD_NUMBER: builtins.int
    STREAM_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    PROPERTIES_FIELD_NUMBER: builtins.int
    TASK_ID_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    TASK_ID_LIST_FIELD_NUMBER: builtins.int
    name: builtins.str
    variant: builtins.str
    description: builtins.str
    entity: builtins.str
    owner: builtins.str
    provider: builtins.str
    task_id: builtins.str
    @property
    def source(self) -> global___NameVariant: ...
    @property
    def created(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def status(self) -> global___ResourceStatus: ...
    @property
    def trainingsets(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NameVariant]: ...
    @property
    def columns(self) -> global___Columns: ...
    @property
    def stream(self) -> global___Stream: ...
    @property
    def tags(self) -> global___Tags: ...
    @property
    def properties(self) -> global___Properties: ...
    @property
    def type(self) -> global___ValueType: ...
    @property
    def task_id_list(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        variant: builtins.str = ...,
        description: builtins.str = ...,
        source: global___NameVariant | None = ...,
        entity: builtins.str = ...,
        created: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        owner: builtins.str = ...,
        provider: builtins.str = ...,
        status: global___ResourceStatus | None = ...,
        trainingsets: collections.abc.Iterable[global___NameVariant] | None = ...,
        columns: global___Columns | None = ...,
        stream: global___Stream | None = ...,
        tags: global___Tags | None = ...,
        properties: global___Properties | None = ...,
        task_id: builtins.str = ...,
        type: global___ValueType | None = ...,
        task_id_list: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["columns", b"columns", "created", b"created", "location", b"location", "properties", b"properties", "source", b"source", "status", b"status", "stream", b"stream", "tags", b"tags", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["columns", b"columns", "created", b"created", "description", b"description", "entity", b"entity", "location", b"location", "name", b"name", "owner", b"owner", "properties", b"properties", "provider", b"provider", "source", b"source", "status", b"status", "stream", b"stream", "tags", b"tags", "task_id", b"task_id", "task_id_list", b"task_id_list", "trainingsets", b"trainingsets", "type", b"type", "variant", b"variant"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["location", b"location"]) -> typing.Literal["columns", "stream"] | None: ...

global___LabelVariant = LabelVariant

@typing.final
class Provider(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    SOFTWARE_FIELD_NUMBER: builtins.int
    TEAM_FIELD_NUMBER: builtins.int
    SERIALIZED_CONFIG_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    SOURCES_FIELD_NUMBER: builtins.int
    FEATURES_FIELD_NUMBER: builtins.int
    TRAININGSETS_FIELD_NUMBER: builtins.int
    LABELS_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    PROPERTIES_FIELD_NUMBER: builtins.int
    name: builtins.str
    description: builtins.str
    type: builtins.str
    software: builtins.str
    team: builtins.str
    serialized_config: builtins.bytes
    @property
    def status(self) -> global___ResourceStatus: ...
    @property
    def sources(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NameVariant]: ...
    @property
    def features(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NameVariant]: ...
    @property
    def trainingsets(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NameVariant]: ...
    @property
    def labels(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NameVariant]: ...
    @property
    def tags(self) -> global___Tags: ...
    @property
    def properties(self) -> global___Properties: ...
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        description: builtins.str = ...,
        type: builtins.str = ...,
        software: builtins.str = ...,
        team: builtins.str = ...,
        serialized_config: builtins.bytes = ...,
        status: global___ResourceStatus | None = ...,
        sources: collections.abc.Iterable[global___NameVariant] | None = ...,
        features: collections.abc.Iterable[global___NameVariant] | None = ...,
        trainingsets: collections.abc.Iterable[global___NameVariant] | None = ...,
        labels: collections.abc.Iterable[global___NameVariant] | None = ...,
        tags: global___Tags | None = ...,
        properties: global___Properties | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["properties", b"properties", "status", b"status", "tags", b"tags"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["description", b"description", "features", b"features", "labels", b"labels", "name", b"name", "properties", b"properties", "serialized_config", b"serialized_config", "software", b"software", "sources", b"sources", "status", b"status", "tags", b"tags", "team", b"team", "trainingsets", b"trainingsets", "type", b"type"]) -> None: ...

global___Provider = Provider

@typing.final
class TrainingSet(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    DEFAULT_VARIANT_FIELD_NUMBER: builtins.int
    VARIANTS_FIELD_NUMBER: builtins.int
    name: builtins.str
    default_variant: builtins.str
    @property
    def status(self) -> global___ResourceStatus: ...
    @property
    def variants(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        status: global___ResourceStatus | None = ...,
        default_variant: builtins.str = ...,
        variants: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["status", b"status"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["default_variant", b"default_variant", "name", b"name", "status", b"status", "variants", b"variants"]) -> None: ...

global___TrainingSet = TrainingSet

@typing.final
class TrainingSetVariant(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    VARIANT_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    OWNER_FIELD_NUMBER: builtins.int
    CREATED_FIELD_NUMBER: builtins.int
    PROVIDER_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    FEATURES_FIELD_NUMBER: builtins.int
    LABEL_FIELD_NUMBER: builtins.int
    LAST_UPDATED_FIELD_NUMBER: builtins.int
    SCHEDULE_FIELD_NUMBER: builtins.int
    FEATURE_LAGS_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    PROPERTIES_FIELD_NUMBER: builtins.int
    TASK_ID_FIELD_NUMBER: builtins.int
    TASK_ID_LIST_FIELD_NUMBER: builtins.int
    name: builtins.str
    variant: builtins.str
    description: builtins.str
    owner: builtins.str
    provider: builtins.str
    schedule: builtins.str
    task_id: builtins.str
    @property
    def created(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def status(self) -> global___ResourceStatus: ...
    @property
    def features(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NameVariant]: ...
    @property
    def label(self) -> global___NameVariant: ...
    @property
    def last_updated(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def feature_lags(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___FeatureLag]: ...
    @property
    def tags(self) -> global___Tags: ...
    @property
    def properties(self) -> global___Properties: ...
    @property
    def task_id_list(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        variant: builtins.str = ...,
        description: builtins.str = ...,
        owner: builtins.str = ...,
        created: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        provider: builtins.str = ...,
        status: global___ResourceStatus | None = ...,
        features: collections.abc.Iterable[global___NameVariant] | None = ...,
        label: global___NameVariant | None = ...,
        last_updated: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        schedule: builtins.str = ...,
        feature_lags: collections.abc.Iterable[global___FeatureLag] | None = ...,
        tags: global___Tags | None = ...,
        properties: global___Properties | None = ...,
        task_id: builtins.str = ...,
        task_id_list: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["created", b"created", "label", b"label", "last_updated", b"last_updated", "properties", b"properties", "status", b"status", "tags", b"tags"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["created", b"created", "description", b"description", "feature_lags", b"feature_lags", "features", b"features", "label", b"label", "last_updated", b"last_updated", "name", b"name", "owner", b"owner", "properties", b"properties", "provider", b"provider", "schedule", b"schedule", "status", b"status", "tags", b"tags", "task_id", b"task_id", "task_id_list", b"task_id_list", "variant", b"variant"]) -> None: ...

global___TrainingSetVariant = TrainingSetVariant

@typing.final
class Entity(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    FEATURES_FIELD_NUMBER: builtins.int
    LABELS_FIELD_NUMBER: builtins.int
    TRAININGSETS_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    PROPERTIES_FIELD_NUMBER: builtins.int
    name: builtins.str
    description: builtins.str
    @property
    def status(self) -> global___ResourceStatus: ...
    @property
    def features(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NameVariant]: ...
    @property
    def labels(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NameVariant]: ...
    @property
    def trainingsets(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NameVariant]: ...
    @property
    def tags(self) -> global___Tags: ...
    @property
    def properties(self) -> global___Properties: ...
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        description: builtins.str = ...,
        status: global___ResourceStatus | None = ...,
        features: collections.abc.Iterable[global___NameVariant] | None = ...,
        labels: collections.abc.Iterable[global___NameVariant] | None = ...,
        trainingsets: collections.abc.Iterable[global___NameVariant] | None = ...,
        tags: global___Tags | None = ...,
        properties: global___Properties | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["properties", b"properties", "status", b"status", "tags", b"tags"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["description", b"description", "features", b"features", "labels", b"labels", "name", b"name", "properties", b"properties", "status", b"status", "tags", b"tags", "trainingsets", b"trainingsets"]) -> None: ...

global___Entity = Entity

@typing.final
class Model(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    FEATURES_FIELD_NUMBER: builtins.int
    LABELS_FIELD_NUMBER: builtins.int
    TRAININGSETS_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    PROPERTIES_FIELD_NUMBER: builtins.int
    name: builtins.str
    description: builtins.str
    @property
    def features(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NameVariant]: ...
    @property
    def labels(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NameVariant]: ...
    @property
    def trainingsets(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NameVariant]: ...
    @property
    def tags(self) -> global___Tags: ...
    @property
    def properties(self) -> global___Properties: ...
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        description: builtins.str = ...,
        features: collections.abc.Iterable[global___NameVariant] | None = ...,
        labels: collections.abc.Iterable[global___NameVariant] | None = ...,
        trainingsets: collections.abc.Iterable[global___NameVariant] | None = ...,
        tags: global___Tags | None = ...,
        properties: global___Properties | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["properties", b"properties", "tags", b"tags"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["description", b"description", "features", b"features", "labels", b"labels", "name", b"name", "properties", b"properties", "tags", b"tags", "trainingsets", b"trainingsets"]) -> None: ...

global___Model = Model

@typing.final
class User(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    FEATURES_FIELD_NUMBER: builtins.int
    LABELS_FIELD_NUMBER: builtins.int
    TRAININGSETS_FIELD_NUMBER: builtins.int
    SOURCES_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    PROPERTIES_FIELD_NUMBER: builtins.int
    name: builtins.str
    @property
    def status(self) -> global___ResourceStatus: ...
    @property
    def features(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NameVariant]: ...
    @property
    def labels(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NameVariant]: ...
    @property
    def trainingsets(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NameVariant]: ...
    @property
    def sources(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NameVariant]: ...
    @property
    def tags(self) -> global___Tags: ...
    @property
    def properties(self) -> global___Properties: ...
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        status: global___ResourceStatus | None = ...,
        features: collections.abc.Iterable[global___NameVariant] | None = ...,
        labels: collections.abc.Iterable[global___NameVariant] | None = ...,
        trainingsets: collections.abc.Iterable[global___NameVariant] | None = ...,
        sources: collections.abc.Iterable[global___NameVariant] | None = ...,
        tags: global___Tags | None = ...,
        properties: global___Properties | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["properties", b"properties", "status", b"status", "tags", b"tags"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["features", b"features", "labels", b"labels", "name", b"name", "properties", b"properties", "sources", b"sources", "status", b"status", "tags", b"tags", "trainingsets", b"trainingsets"]) -> None: ...

global___User = User

@typing.final
class Source(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    DEFAULT_VARIANT_FIELD_NUMBER: builtins.int
    VARIANTS_FIELD_NUMBER: builtins.int
    name: builtins.str
    default_variant: builtins.str
    @property
    def status(self) -> global___ResourceStatus: ...
    @property
    def variants(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        status: global___ResourceStatus | None = ...,
        default_variant: builtins.str = ...,
        variants: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["status", b"status"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["default_variant", b"default_variant", "name", b"name", "status", b"status", "variants", b"variants"]) -> None: ...

global___Source = Source

@typing.final
class SourceVariant(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    VARIANT_FIELD_NUMBER: builtins.int
    TRANSFORMATION_FIELD_NUMBER: builtins.int
    PRIMARYDATA_FIELD_NUMBER: builtins.int
    OWNER_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    PROVIDER_FIELD_NUMBER: builtins.int
    CREATED_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    TABLE_FIELD_NUMBER: builtins.int
    TRAININGSETS_FIELD_NUMBER: builtins.int
    FEATURES_FIELD_NUMBER: builtins.int
    LABELS_FIELD_NUMBER: builtins.int
    LAST_UPDATED_FIELD_NUMBER: builtins.int
    SCHEDULE_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    PROPERTIES_FIELD_NUMBER: builtins.int
    TASK_ID_FIELD_NUMBER: builtins.int
    MAX_JOB_DURATION_FIELD_NUMBER: builtins.int
    TASK_ID_LIST_FIELD_NUMBER: builtins.int
    name: builtins.str
    variant: builtins.str
    owner: builtins.str
    description: builtins.str
    provider: builtins.str
    table: builtins.str
    schedule: builtins.str
    task_id: builtins.str
    @property
    def transformation(self) -> global___Transformation: ...
    @property
    def primaryData(self) -> global___PrimaryData: ...
    @property
    def created(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def status(self) -> global___ResourceStatus: ...
    @property
    def trainingsets(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NameVariant]: ...
    @property
    def features(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NameVariant]: ...
    @property
    def labels(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NameVariant]: ...
    @property
    def last_updated(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def tags(self) -> global___Tags: ...
    @property
    def properties(self) -> global___Properties: ...
    @property
    def max_job_duration(self) -> google.protobuf.duration_pb2.Duration: ...
    @property
    def task_id_list(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        variant: builtins.str = ...,
        transformation: global___Transformation | None = ...,
        primaryData: global___PrimaryData | None = ...,
        owner: builtins.str = ...,
        description: builtins.str = ...,
        provider: builtins.str = ...,
        created: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        status: global___ResourceStatus | None = ...,
        table: builtins.str = ...,
        trainingsets: collections.abc.Iterable[global___NameVariant] | None = ...,
        features: collections.abc.Iterable[global___NameVariant] | None = ...,
        labels: collections.abc.Iterable[global___NameVariant] | None = ...,
        last_updated: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        schedule: builtins.str = ...,
        tags: global___Tags | None = ...,
        properties: global___Properties | None = ...,
        task_id: builtins.str = ...,
        max_job_duration: google.protobuf.duration_pb2.Duration | None = ...,
        task_id_list: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["created", b"created", "definition", b"definition", "last_updated", b"last_updated", "max_job_duration", b"max_job_duration", "primaryData", b"primaryData", "properties", b"properties", "status", b"status", "tags", b"tags", "transformation", b"transformation"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["created", b"created", "definition", b"definition", "description", b"description", "features", b"features", "labels", b"labels", "last_updated", b"last_updated", "max_job_duration", b"max_job_duration", "name", b"name", "owner", b"owner", "primaryData", b"primaryData", "properties", b"properties", "provider", b"provider", "schedule", b"schedule", "status", b"status", "table", b"table", "tags", b"tags", "task_id", b"task_id", "task_id_list", b"task_id_list", "trainingsets", b"trainingsets", "transformation", b"transformation", "variant", b"variant"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["definition", b"definition"]) -> typing.Literal["transformation", "primaryData"] | None: ...

global___SourceVariant = SourceVariant

@typing.final
class Transformation(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SQLTRANSFORMATION_FIELD_NUMBER: builtins.int
    DFTRANSFORMATION_FIELD_NUMBER: builtins.int
    KUBERNETES_ARGS_FIELD_NUMBER: builtins.int
    @property
    def SQLTransformation(self) -> global___SQLTransformation: ...
    @property
    def DFTransformation(self) -> global___DFTransformation: ...
    @property
    def kubernetes_args(self) -> global___KubernetesArgs: ...
    def __init__(
        self,
        *,
        SQLTransformation: global___SQLTransformation | None = ...,
        DFTransformation: global___DFTransformation | None = ...,
        kubernetes_args: global___KubernetesArgs | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["DFTransformation", b"DFTransformation", "SQLTransformation", b"SQLTransformation", "args", b"args", "kubernetes_args", b"kubernetes_args", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["DFTransformation", b"DFTransformation", "SQLTransformation", b"SQLTransformation", "args", b"args", "kubernetes_args", b"kubernetes_args", "type", b"type"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["args", b"args"]) -> typing.Literal["kubernetes_args"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["type", b"type"]) -> typing.Literal["SQLTransformation", "DFTransformation"] | None: ...

global___Transformation = Transformation

@typing.final
class KubernetesResourceSpecs(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CPU_REQUEST_FIELD_NUMBER: builtins.int
    CPU_LIMIT_FIELD_NUMBER: builtins.int
    MEMORY_REQUEST_FIELD_NUMBER: builtins.int
    MEMORY_LIMIT_FIELD_NUMBER: builtins.int
    cpu_request: builtins.str
    cpu_limit: builtins.str
    memory_request: builtins.str
    memory_limit: builtins.str
    def __init__(
        self,
        *,
        cpu_request: builtins.str = ...,
        cpu_limit: builtins.str = ...,
        memory_request: builtins.str = ...,
        memory_limit: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["cpu_limit", b"cpu_limit", "cpu_request", b"cpu_request", "memory_limit", b"memory_limit", "memory_request", b"memory_request"]) -> None: ...

global___KubernetesResourceSpecs = KubernetesResourceSpecs

@typing.final
class KubernetesArgs(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DOCKER_IMAGE_FIELD_NUMBER: builtins.int
    SPECS_FIELD_NUMBER: builtins.int
    docker_image: builtins.str
    @property
    def specs(self) -> global___KubernetesResourceSpecs: ...
    def __init__(
        self,
        *,
        docker_image: builtins.str = ...,
        specs: global___KubernetesResourceSpecs | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["specs", b"specs"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["docker_image", b"docker_image", "specs", b"specs"]) -> None: ...

global___KubernetesArgs = KubernetesArgs

@typing.final
class SQLTransformation(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    QUERY_FIELD_NUMBER: builtins.int
    SOURCE_FIELD_NUMBER: builtins.int
    query: builtins.str
    @property
    def source(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NameVariant]: ...
    def __init__(
        self,
        *,
        query: builtins.str = ...,
        source: collections.abc.Iterable[global___NameVariant] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["query", b"query", "source", b"source"]) -> None: ...

global___SQLTransformation = SQLTransformation

@typing.final
class DFTransformation(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    QUERY_FIELD_NUMBER: builtins.int
    INPUTS_FIELD_NUMBER: builtins.int
    SOURCE_TEXT_FIELD_NUMBER: builtins.int
    query: builtins.bytes
    source_text: builtins.str
    @property
    def inputs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___NameVariant]: ...
    def __init__(
        self,
        *,
        query: builtins.bytes = ...,
        inputs: collections.abc.Iterable[global___NameVariant] | None = ...,
        source_text: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["inputs", b"inputs", "query", b"query", "source_text", b"source_text"]) -> None: ...

global___DFTransformation = DFTransformation

@typing.final
class PrimaryData(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TABLE_FIELD_NUMBER: builtins.int
    @property
    def table(self) -> global___PrimarySQLTable: ...
    def __init__(
        self,
        *,
        table: global___PrimarySQLTable | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["location", b"location", "table", b"table"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["location", b"location", "table", b"table"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["location", b"location"]) -> typing.Literal["table"] | None: ...

global___PrimaryData = PrimaryData

@typing.final
class PrimarySQLTable(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    name: builtins.str
    def __init__(
        self,
        *,
        name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["name", b"name"]) -> None: ...

global___PrimarySQLTable = PrimarySQLTable

@typing.final
class Tags(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TAG_FIELD_NUMBER: builtins.int
    @property
    def tag(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        tag: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["tag", b"tag"]) -> None: ...

global___Tags = Tags

@typing.final
class Property(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STRING_VALUE_FIELD_NUMBER: builtins.int
    string_value: builtins.str
    def __init__(
        self,
        *,
        string_value: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["string_value", b"string_value", "value", b"value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["string_value", b"string_value", "value", b"value"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["value", b"value"]) -> typing.Literal["string_value"] | None: ...

global___Property = Property

@typing.final
class Properties(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class PropertyEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___Property: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: global___Property | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    PROPERTY_FIELD_NUMBER: builtins.int
    @property
    def property(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___Property]: ...
    def __init__(
        self,
        *,
        property: collections.abc.Mapping[builtins.str, global___Property] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["property", b"property"]) -> None: ...

global___Properties = Properties

@typing.final
class StreamingFeatureVariant(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    VARIANT_FIELD_NUMBER: builtins.int
    ENTITY_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    TS_FIELD_NUMBER: builtins.int
    name: builtins.str
    variant: builtins.str
    entity: builtins.str
    value: builtins.str
    @property
    def ts(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        variant: builtins.str = ...,
        entity: builtins.str = ...,
        value: builtins.str = ...,
        ts: google.protobuf.timestamp_pb2.Timestamp | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["ts", b"ts"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["entity", b"entity", "name", b"name", "ts", b"ts", "value", b"value", "variant", b"variant"]) -> None: ...

global___StreamingFeatureVariant = StreamingFeatureVariant

@typing.final
class StreamingLabelVariant(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    VARIANT_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    ENTITY_FIELD_NUMBER: builtins.int
    TS_FIELD_NUMBER: builtins.int
    name: builtins.str
    variant: builtins.str
    value: builtins.str
    entity: builtins.str
    @property
    def ts(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        variant: builtins.str = ...,
        value: builtins.str = ...,
        entity: builtins.str = ...,
        ts: google.protobuf.timestamp_pb2.Timestamp | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["ts", b"ts"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["entity", b"entity", "name", b"name", "ts", b"ts", "value", b"value", "variant", b"variant"]) -> None: ...

global___StreamingLabelVariant = StreamingLabelVariant



import os


def is_enabled(feature_name, default=False):
    """
    Returns True if the feature is enabled, currently using an environment variable.
    Interprets the environment variable as a boolean if it's a valid representation, else defaults to the specified default.
    """
    value = os.getenv(feature_name)
    if value is not None:
        return value.lower() == "true"
    return default



import sys
import time
from typing import List, Tuple, Type

from dataclasses import dataclass
from rich.console import Console
from rich.live import Live
from rich.table import Table
from rich.text import Text

from featureform.grpc_client import GrpcClient
from featureform.resources import (
    ErrorInfo,
    FeatureVariant,
    LabelVariant,
    OnDemandFeatureVariant,
    Provider,
    Resource,
    SourceVariant,
    StreamFeature,
    StreamLabel,
    TrainingSetVariant,
)

# maximum number of dots printing when featureform apply for Running...
MAX_NUM_RUNNING_DOTS = 10
SECONDS_BETWEEN_STATUS_CHECKS = 2


def display_statuses(grpc_client: GrpcClient, resources: List[Resource], verbose=False):
    StatusDisplayer(grpc_client, resources, verbose=verbose).display()


@dataclass
class DisplayStatus:
    resource_type: Type
    name: str
    variant: str
    status: str = "NO_STATUS"
    error: str = None
    has_health_check: bool = False

    def is_finished(self) -> bool:
        return (
            self.status == "READY"
            or self.status == "FAILED"
            or (
                self.resource_type is Provider
                and self.status == "NO_STATUS"
                and not self.has_health_check
            )  # Provider is a special case
        )

    @staticmethod
    def from_resource(resource: Resource):
        variant = getattr(resource, "variant", "")
        return DisplayStatus(
            resource_type=type(resource),
            name=resource.name,
            variant=variant,
            status=resource.status,
            error=resource.error,
            has_health_check=bool(getattr(resource, "has_health_check", False)),
        )


class StatusDisplayer:
    did_error: bool = False
    RESOURCE_TYPES_TO_CHECK = {
        FeatureVariant,
        OnDemandFeatureVariant,
        TrainingSetVariant,
        LabelVariant,
        SourceVariant,
        StreamFeature,
        StreamLabel,
        Provider,
    }

    STATUS_TO_COLOR = {
        "READY": "green",
        "CREATED": "green",
        "PENDING": "white",
        "NO_STATUS": "white",
        "FAILED": "red",
        "RUNNING": "yellow"
    }

    def __init__(
        self, grpc_client: GrpcClient, resources: List[Resource], verbose=False
    ):
        self.verbose = verbose
        filtered_resources = filter(
            lambda r: any(
                isinstance(r, resource_type)
                for resource_type in self.RESOURCE_TYPES_TO_CHECK
            ),
            resources,
        )
        self.grpc_client = grpc_client

        # A more intuitive way to is to store OrderedDict[Resource, DisplayStatus] but you can't hash Resource easily
        self.resource_to_status_list: List[Tuple[Resource, DisplayStatus]] = []

        for r in filtered_resources:
            self.resource_to_status_list.append((r, DisplayStatus.from_resource(r)))

    def update_display_statuses(self):
        for resource, display_status in self.resource_to_status_list:
            if resource.name == "local-mode":
                continue
            if not display_status.is_finished():
                r = resource.get(self.grpc_client)
                server_status = r.server_status
                display_status.status = server_status.status
                if server_status.error_info is not None:
                    display_status.error = self._format_error_info(
                        server_status.error_info
                    )
                else:
                    display_status.error = r.error

    @staticmethod
    def _format_error_info(error_info: ErrorInfo):
        message = error_info.message
        reason = error_info.reason
        metadata = error_info.metadata

        formatted_metadata = "\n".join(
            [f"{key}: {value}" for key, value in metadata.items()]
        )

        return f"{reason}: {message}\n{formatted_metadata}"

    def all_statuses_finished(self) -> bool:
        return all(status.is_finished() for _, status in self.resource_to_status_list)

    def create_error_message(self):
        message = ""
        for _, status in self.resource_to_status_list:
            name = status.name
            message += f"{name}: {status.status} - {status.error}\n"
        return message

    def display(self):
        if not self.resource_to_status_list:
            return

        print()
        console = Console()
        with Live(console=console, auto_refresh=True, screen=False) as live:
            i = 0
            while True:
                self.update_display_statuses()
                finished_running = self.all_statuses_finished()

                dots = "." * (1 + i % MAX_NUM_RUNNING_DOTS)

                title = (
                    f"[green]COMPLETED[/]"
                    if finished_running
                    else f"[yellow]RUNNING{dots}[/]"
                )
                table = Table(
                    title=title,
                    title_justify="left",
                    show_header=True,
                    header_style="bold",
                    box=None,
                )

                no_table = False
                if len(self.resource_to_status_list) == 1:
                    status = self.resource_to_status_list[0][1]
                    if status.name == "local-mode":
                        no_table = True

                if not no_table:
                    table.add_column("Resource Type", width=25)
                    table.add_column("Name (Variant)", width=50, no_wrap=True)
                    table.add_column("Status", width=10)
                    table.add_column("Error", style="red")

                    for resource, status in self.resource_to_status_list:
                        error = f" {status.error}" if status.error else ""
                        if status.name == "local-mode":
                            continue
                        resource_type = status.resource_type.__name__
                        if (
                            isinstance(resource, SourceVariant)
                            and resource.is_transformation_type()
                        ):
                            resource_type = "Transformation"
                        name = status.name
                        status_text = (
                            status.status
                            if status.resource_type is not Provider
                            or status.has_health_check
                            else "CREATED"
                        )

                        table.add_row(
                            Text(resource_type),
                            Text(f"{name} ({status.variant})"),
                            Text(status_text, style=self.STATUS_TO_COLOR[status_text]),
                            Text(error, style="red"),
                        )

                live.update(table)
                live.refresh()

                if finished_running:
                    # This block is used for testing
                    # Tests check for both stderr and an exception
                    # If we don't throw an exception, then tests will pass even when things fail to register
                    # We also print all the error messages because the table does not get saved when
                    # capturing stdout/stderr
                    if self.verbose and self.did_error:
                        statuses = self.create_error_message()
                        sys.tracebacklimit = 0
                        raise Exception("Some resources failed to create\n" + statuses)
                    break

                i += 1
                time.sleep(SECONDS_BETWEEN_STATUS_CHECKS)



import os

import numpy as np
import pytest

from featureform.proto import serving_pb2
from featureform.serving import Dataset

real_path = os.path.realpath(__file__)
dir_path = os.path.dirname(real_path)


def response(
    req_type, iterator_done, batch_size
) -> serving_pb2.BatchTrainTestSplitResponse:
    if req_type == serving_pb2.RequestType.INITIALIZE:
        return serving_pb2.BatchTrainTestSplitResponse(
            request_type=req_type, initialized=True
        )
    elif req_type == serving_pb2.RequestType.TRAINING:
        label_value = "train"
    elif req_type == serving_pb2.RequestType.TEST:
        label_value = "test"

    row = serving_pb2.TrainingDataRow(
        features=[
            serving_pb2.Value(str_value="f1"),
            serving_pb2.Value(int_value=1),
            serving_pb2.Value(bool_value=False),
        ],
        label=serving_pb2.Value(str_value=label_value),
    )

    return serving_pb2.BatchTrainTestSplitResponse(
        request_type=req_type,
        data=serving_pb2.TrainingDataRows(rows=[row] * batch_size),
        iterator_done=iterator_done,
    )


class MockGrpcStub:
    def __init__(self, num_rows, batch_size=1):
        self.num_rows = num_rows
        self.train_rows = 0
        self.test_rows = 0
        self.batch_size = batch_size

    def TrainTestSplit(self, iterator) -> serving_pb2.BatchTrainTestSplitResponse:
        rows_to_return = self.batch_size
        for value in iterator:
            iterator_done = False
            if value.request_type == serving_pb2.RequestType.TRAINING:
                self.train_rows += self.batch_size

                if self.train_rows > self.num_rows:
                    rows_to_return = self.num_rows - (self.train_rows - self.batch_size)
                    iterator_done = True
            elif value.request_type == serving_pb2.RequestType.TEST:
                self.test_rows += self.batch_size

                if self.test_rows > self.num_rows:
                    rows_to_return = self.num_rows - (self.test_rows - self.batch_size)
                    iterator_done = True
            yield response(value.request_type, iterator_done, rows_to_return)


class MockStream:
    name = None
    version = None
    model = None
    _stub = None

    def __init__(self, name, version, num_rows, batch_size=1):
        self.name = name
        self.version = version
        self._stub = MockGrpcStub(num_rows, batch_size)


@pytest.mark.parametrize(
    "kwargs, should_fail",
    [
        (
            {
                "test_size": 0.5,
                "train_size": 0.5,
                "shuffle": True,
                "random_state": None,
            },
            False,
        ),
        (
            {
                "test_size": -0.5,
                "train_size": 0.5,
                "shuffle": True,
                "random_state": None,
            },
            True,
        ),
        (
            {
                "test_size": 0.5,
                "train_size": -0.5,
                "shuffle": True,
                "random_state": None,
            },
            True,
        ),
        ({"test_size": 0.5, "shuffle": True, "random_state": None}, False),
        ({"test_size": 0.5, "random_state": None}, False),
        ({"test_size": 0.5}, False),
        ({"test_size": 0.5, "random_state": 3}, False),
        ({"train_size": 0.5, "shuffle": True, "random_state": None}, False),
        (
            {
                "test_size": 0.5,
                "train_size": 0.1,
                "shuffle": True,
                "random_state": None,
            },
            True,
        ),
    ],
)
def test_initialization(kwargs, should_fail):
    train_test_stream = MockStream("name", "variant", 1)
    dataset = Dataset(train_test_stream)
    if should_fail:
        with pytest.raises(Exception):
            dataset.train_test_split(**kwargs)
    else:
        dataset.train_test_split(**kwargs)


def test_train_test_split():
    train_test_stream = MockStream("name", "variant", 1)
    dataset = Dataset(train_test_stream)
    train, test = dataset.train_test_split(
        test_size=0.5, train_size=0.5, shuffle=True, random_state=None
    )

    for features, label in train:
        assert np.array_equal(features, np.array([["f1", 1, False]], dtype="O"))
        assert np.array_equal(label, np.array(["train"]))

    for features, label in test:
        assert np.array_equal(features, np.array([["f1", 1, False]], dtype="O"))
        assert np.array_equal(label, np.array(["test"]))


def test_train_test_batch():
    train_test_stream = MockStream("name", "variant", 10, batch_size=5)
    dataset = Dataset(train_test_stream)
    train, test = dataset.train_test_split(
        test_size=0.5, train_size=0.5, shuffle=True, random_state=None, batch_size=5
    )
    i = 0
    for features, label in train:
        if i > 1:
            break  # Ensures we only check the first two batches

        assert np.array_equal(
            features,
            np.array(
                [
                    ["f1", 1, False],
                    ["f1", 1, False],
                    ["f1", 1, False],
                    ["f1", 1, False],
                    ["f1", 1, False],
                ],
                dtype="O",
            ),
        )
        assert np.array_equal(
            label, np.array(["train", "train", "train", "train", "train"])
        )
        i += 1

    i = 0
    for features, label in test:
        if i > 1:
            break
        assert np.array_equal(
            features,
            np.array(
                [
                    ["f1", 1, False],
                    ["f1", 1, False],
                    ["f1", 1, False],
                    ["f1", 1, False],
                    ["f1", 1, False],
                ],
                dtype="O",
            ),
        )
        assert np.array_equal(label, np.array(["test", "test", "test", "test", "test"]))
        i += 1


def test_train_test_partial_batch():
    train_test_stream = MockStream("name", "variant", 3, batch_size=5)
    dataset = Dataset(train_test_stream)
    train, test = dataset.train_test_split(
        test_size=0.5, train_size=0.5, shuffle=True, random_state=None, batch_size=5
    )
    i = 0
    for features, label in train:
        if i > 1:
            break  # Ensures we only check the first two batches

        assert np.array_equal(
            features,
            np.array(
                [
                    ["f1", 1, False],
                    ["f1", 1, False],
                    ["f1", 1, False],
                ],
                dtype="O",
            ),
        )
        assert np.array_equal(label, np.array(["train", "train", "train"]))
        i += 1

    i = 0
    for features, label in test:
        if i > 1:
            break
        assert np.array_equal(
            features,
            np.array(
                [
                    ["f1", 1, False],
                    ["f1", 1, False],
                    ["f1", 1, False],
                ],
                dtype="O",
            ),
        )
        assert np.array_equal(label, np.array(["test", "test", "test"]))
        i += 1



from featureform.proto import metadata_pb2
import grpc
from featureform.proto import metadata_pb2_grpc as ff_grpc
from .format import *


def get_user_info(stub, name):
    searchName = metadata_pb2.Name(name=name)
    try:
        for user in stub.GetUsers(iter([searchName])):
            format_rows("USER NAME: ", user.name)
            format_tags_and_properties(user.tags, user.properties)
            format_pg()
            format_rows("NAME", "VARIANT", "TYPE")
            for f in user.features:
                format_rows(f.name, f.variant, "feature")
            for l in user.labels:
                format_rows(l.name, l.variant, "label")
            for t in user.trainingsets:
                format_rows(t.name, t.variant, "training set")
            for s in user.sources:
                format_rows(s.name, s.variant, "source")
            format_pg()
            return user
    except grpc._channel._MultiThreadedRendezvous:
        print("User not found.")


def get_entity_info(stub, name):
    searchName = metadata_pb2.Name(name=name)
    try:
        for x in stub.GetEntities(iter([searchName])):
            format_rows(
                [
                    ("ENTITY NAME: ", x.name),
                    (
                        "STATUS: ",
                        x.status.Status._enum_type.values[x.status.status].name,
                    ),
                ]
            )
            format_tags_and_properties(x.tags, x.properties)
            format_pg()
            format_rows("NAME", "VARIANT", "TYPE")
            for f in x.features:
                format_rows(f.name, f.variant, "feature")
            for l in x.labels:
                format_rows(l.name, l.variant, "label")
            for t in x.trainingsets:
                format_rows(t.name, t.variant, "training set")
            format_pg()
            return x
    except grpc._channel._MultiThreadedRendezvous:
        print("Entity not found.")


def get_resource_info(stub, resource_type, name):
    stub_get_functions = {
        "feature": stub.GetFeatures,
        "label": stub.GetLabels,
        "source": stub.GetSources,
        "trainingset": stub.GetTrainingSets,
        "training-set": stub.GetTrainingSets,
        "model": stub.GetModels,
    }

    searchName = metadata_pb2.Name(name=name)
    try:
        for x in stub_get_functions[resource_type](iter([searchName])):
            rows = [("NAME: ", x.name)]
            if hasattr(x, '"status'):
                rows.append(
                    (
                        "STATUS: ",
                        x.status.Status._enum_type.values[x.status.status].name,
                    )
                )
            format_rows(rows)
            if hasattr(x, "default_variant"):
                format_pg("VARIANTS:")
                format_rows(x.default_variant, "default")
                for v in x.variants:
                    if v != x.default_variant:
                        format_rows(v, "")
                format_pg()
            return x
    except grpc._channel._MultiThreadedRendezvous:
        print(f"{resource_type} not found.")


def get_feature_variant_info(stub, name, variant):
    searchNameVariant = metadata_pb2.NameVariant(name=name, variant=variant)
    try:
        for x in stub.GetFeatureVariants(iter([searchNameVariant])):
            status = x.status.Status._enum_type.values[x.status.status].name
            rows = [
                ("NAME: ", x.name),
                ("VARIANT: ", x.variant),
                ("TYPE:", x.type),
                ("ENTITY:", x.entity),
                ("OWNER:", x.owner),
                ("DESCRIPTION:", x.description),
                ("PROVIDER:", x.provider),
                ("STATUS: ", x.status.Status._enum_type.values[x.status.status].name),
            ]
            if status == "FAILED":
                rows.append(("ERROR: ", x.status.error_message))
            format_rows(rows)
            format_tags_and_properties(x.tags, x.properties)
            format_pg("SOURCE: ")
            format_rows([("NAME", "VARIANT"), (x.source.name, x.source.variant)])
            format_pg("TRAINING SETS:")
            format_rows("NAME", "VARIANT")
            for t in x.trainingsets:
                format_rows(t.name, t.variant)
            format_pg()
            return x
    except grpc._channel._MultiThreadedRendezvous:
        print("Feature variant not found.")


def get_label_variant_info(stub, name, variant):
    searchNameVariant = metadata_pb2.NameVariant(name=name, variant=variant)
    try:
        for x in stub.GetLabelVariants(iter([searchNameVariant])):
            status = x.status.Status._enum_type.values[x.status.status].name
            rows = [
                ("NAME: ", x.name),
                ("VARIANT: ", x.variant),
                ("TYPE:", x.type),
                ("ENTITY:", x.entity),
                ("OWNER:", x.owner),
                ("DESCRIPTION:", x.description),
                ("PROVIDER:", x.provider),
                ("STATUS: ", x.status.Status._enum_type.values[x.status.status].name),
            ]
            if status == "FAILED":
                rows.append(("ERROR: ", x.status.error_message))
            format_rows(rows)
            format_tags_and_properties(x.tags, x.properties)
            format_pg("SOURCE: ")
            format_rows([("NAME", "VARIANT"), (x.source.name, x.source.variant)])
            format_pg("TRAINING SETS:")
            format_rows("NAME", "VARIANT")
            for t in x.trainingsets:
                format_rows(t.name, t.variant)
            format_pg()
            return x
    except grpc._channel._MultiThreadedRendezvous:
        print("Label variant not found.")


def get_source_variant_info(stub, name, variant):
    searchNameVariant = metadata_pb2.NameVariant(name=name, variant=variant)
    try:
        for x in stub.GetSourceVariants(iter([searchNameVariant])):
            status = x.status.Status._enum_type.values[x.status.status].name
            rows = [
                ("NAME: ", x.name),
                ("VARIANT: ", x.variant),
                ("OWNER:", x.owner),
                ("DESCRIPTION:", x.description),
                ("PROVIDER:", x.provider),
                ("TABLE:", x.table),
                ("STATUS: ", x.status.Status._enum_type.values[x.status.status].name),
            ]
            if status == "FAILED":
                rows.append(("ERROR: ", x.status.error_message))
            format_rows(rows)
            format_tags_and_properties(x.tags, x.properties)
            format_pg("DEFINITION:")
            print("TRANSFORMATION")
            print(x.transformation.SQLTransformation.query)
            format_pg("SOURCES")
            format_rows("NAME", "VARIANT")
            for s in x.transformation.SQLTransformation.source:
                format_rows(s.name, s.variant)
            format_pg("PRIMARY DATA")
            print(x.primaryData.table.name)
            print("FEATURES:")
            format_rows("NAME", "VARIANT")
            for t in x.features:
                format_rows(t.name, t.variant)
            format_pg("LABELS:")
            format_rows("NAME", "VARIANT")
            for t in x.labels:
                format_rows(t.name, t.variant)
            format_pg("TRAINING SETS:")
            format_rows("NAME", "VARIANT")
            for t in x.trainingsets:
                format_rows(t.name, t.variant)
            format_pg()
            return x
    except grpc._channel._MultiThreadedRendezvous:
        print("Source variant not found.")


def get_training_set_variant_info(stub, name, variant):
    searchNameVariant = metadata_pb2.NameVariant(name=name, variant=variant)
    try:
        for x in stub.GetTrainingSetVariants(iter([searchNameVariant])):
            status = x.status.Status._enum_type.values[x.status.status].name
            rows = [
                ("NAME: ", x.name),
                ("VARIANT: ", x.variant),
                ("OWNER:", x.owner),
                ("DESCRIPTION:", x.description),
                ("PROVIDER:", x.provider),
                ("STATUS: ", x.status.Status._enum_type.values[x.status.status].name),
            ]
            if status == "FAILED":
                rows.append(("ERROR: ", x.status.error_message))
            format_rows(rows)
            format_tags_and_properties(x.tags, x.properties)
            format_pg("LABEL: ")
            format_rows([("NAME", "VARIANT"), (x.label.name, x.label.variant)])
            format_pg("FEATURES:")
            format_rows("NAME", "VARIANT")
            for f in x.features:
                format_rows(f.name, f.variant)
            format_pg()
            return x
    except grpc._channel._MultiThreadedRendezvous:
        print("Training set variant not found.")


def get_provider_info(stub, name):
    searchName = metadata_pb2.Name(name=name)
    try:
        for x in stub.GetProviders(iter([searchName])):
            status = x.status.Status._enum_type.values[x.status.status].name
            rows = [
                ("NAME: ", x.name),
                ("DESCRIPTION: ", x.description),
                ("TYPE: ", x.type),
                ("SOFTWARE: ", x.software),
                ("TEAM: ", x.team),
                ("STATUS: ", x.status.Status._enum_type.values[x.status.status].name),
            ]
            if status == "FAILED":
                rows.append(("ERROR: ", x.status.error_message))
            format_rows(rows)
            format_tags_and_properties(x.tags, x.properties)
            format_pg("SOURCES:")
            format_rows("NAME", "VARIANT")
            for s in x.sources:
                format_rows(s.name, s.variant)
            format_pg("FEATURES:")
            format_rows("NAME", "VARIANT")
            for f in x.features:
                format_rows(f.name, f.variant)
            format_pg("LABELS:")
            format_rows("NAME", "VARIANT")
            for l in x.labels:
                format_rows(l.name, l.variant)
            format_pg("TRAINING SETS:")
            format_rows("NAME", "VARIANT")
            for t in x.trainingsets:
                format_rows(t.name, t.variant)
            format_pg()
            return x
    except grpc._channel._MultiThreadedRendezvous:
        print("Provider not found.")


def format_tags_and_properties(tags, properties):
    if len(tags.tag):
        format_rows("TAGS:", ", ".join(tags.tag))
    if len(properties.property.items()):
        format_rows(
            "PROPERTIES:",
            ", ".join(
                [f"{k}:{v.string_value}" for (k, v) in properties.property.items()]
            ),
        )



import pytest

from featureform.resources import (
    DatabricksCredentials,
    EMRCredentials,
    AWSStaticCredentials,
)


@pytest.mark.parametrize(
    "username,password,host,token,cluster_id",
    [
        # either use username-password pair or host-token pair
        (
            "",
            "",
            "host_xyz",
            "dapiabcdefghijklmnopqrstuvwxyz123456-6",
            "abcd-123def-ghijklmn",
        ),
        # valid token and cluster id
        pytest.param(
            "john",
            "abc123",
            "host_xyz",
            "dapiabcdefghijklmnopqrstuvwxyz12345-6",
            "abcd-123def-ghijklmn",
            marks=pytest.mark.xfail,
        ),
        # valid token w/o hyphenated suffix and cluster id
        pytest.param(
            "",
            "",
            "host_xyz",
            "dapiabcdefghijklmnopqrstuvwxyz123456",
            "abcd-123def-ghijklmn",
        ),
        # cluster id should always be provided
        ("john", "abc123", "", "", "abcd-123def-ghijklmn"),
        pytest.param(
            "john",
            "abc123",
            "",
            "",
            "",
            marks=pytest.mark.xfail,
        ),
        # token and cluster id are empty
        pytest.param("", "", "a", "", "", marks=pytest.mark.xfail),  # invalid case
        # invalid token id
        pytest.param(
            "",
            "",
            "host_xyz",
            "dainvalid_token",
            "abcd-123def-ghijklmn",
            marks=pytest.mark.xfail,
        ),
        pytest.param(
            "",
            "",
            "host_xyz",
            "dainvalid#$%_token!$%",
            "abcd-123def-ghijklmn",
            marks=pytest.mark.xfail,
        ),
        # invalid cluster id
        pytest.param(
            "",
            "",
            "host_xyz",
            "dapiabcdefghijklmnopqrstuvwxyz12345-6",
            "abc-1234def-ghijklmnopq",
            marks=pytest.mark.xfail,
        ),
        pytest.param(
            "",
            "",
            "host_xyz",
            "dapiabcdefghijklmnopqrstuvwxyz12345-6",
            "abc-!@#4def-ghijklmnopq",
            marks=pytest.mark.xfail,
        ),
        pytest.param("", "", "", "", "abcd-123def-ghijklmn", marks=pytest.mark.xfail),
    ],
)
def test_databricks_credentials(username, password, host, token, cluster_id):
    databricks = DatabricksCredentials(
        username=username,
        password=password,
        host=host,
        token=token,
        cluster_id=cluster_id,
    )

    expected_config = {
        "Username": username,
        "Password": password,
        "Host": host,
        "Token": token,
        "Cluster": cluster_id,
    }

    assert databricks.type() == "DATABRICKS"
    assert databricks.config() == expected_config


@pytest.mark.parametrize(
    "aws_access_key_id,aws_secret_access_key,emr_cluster_id,emr_cluster_region",
    [
        ("a", "b", "c", "d"),
        pytest.param("", "", "a", "b", marks=pytest.mark.xfail),
        pytest.param("", "", "", "", marks=pytest.mark.xfail),
    ],
)
def test_emr_credentials(
    aws_access_key_id, aws_secret_access_key, emr_cluster_id, emr_cluster_region
):
    emr = EMRCredentials(
        emr_cluster_id=emr_cluster_id,
        emr_cluster_region=emr_cluster_region,
        credentials=AWSStaticCredentials(
            access_key=aws_access_key_id, secret_key=aws_secret_access_key
        ),
    )

    expected_config = {
        "ClusterName": emr_cluster_id,
        "ClusterRegion": emr_cluster_region,
        "Credentials": {
            "AccessKeyId": "a",
            "SecretKey": "b",
            "Type": "AWS_STATIC_CREDENTIALS",
        },
    }

    assert emr.type() == "EMR"
    assert emr.config() == expected_config
